#ifndef _OWLPS_POSITIONING_AREA_HH_
#define _OWLPS_POSITIONING_AREA_HH_

class Building ;

#include "point3d.hh"

#include <string>
#include <ostream>

/// Represents an homogeneous area (typically a room) of a Building
class Area
{
protected:
  Building *building ; ///< The Building in which the Area is
  std::string name ;
  Point3D p_min ; ///< First coordinate of the Area
  Point3D p_max ; ///< Second coordinate of the Area

  void reorder_coordinates(void) ; ///< Reorders #p_min and #p_max

public:
  Area(const Building *_building = NULL,
       const std::string &_name = "Unnamed area",
       const Point3D &p1 = Point3D(0,0,0),
       const Point3D &p2 = Point3D(0,0,0)) ;

  Area(const Area &source):
    building(source.building), name(source.name),
    p_min(source.p_min), p_max(source.p_max) {}

  ~Area(void) {}

  /** @name Read accessors */
  //@{
  Building* get_building(void) const ;
  const std::string& get_name(void) const ;
  const Point3D& get_p_min(void) const ;
  const Point3D& get_p_max(void) const ;
  //@}

  /** @name Write accessors */
  //@{
  void set_building(const Building *_building) ;
  void set_name(const std::string &_name) ;
  /// Sets the coordinates of the Area and reorder them
  void set_coordinates(const Point3D &p1, const Point3D &p2) ;
  //@}

  /** @name Operations */
  //@{
  /// Checks if a Point3D is into the Area
  bool contains_point(const Point3D &p) const ;
  //@}

  /** @name Operators */
  //@{
  Area& operator=(const Area &source) ;
  bool operator==(const Area &source) const ;
  bool operator!=(const Area &source) const ;
  //@}

  /// Displays an Area
  friend std::ostream &operator<<(std::ostream &os, const Area &a) ;
} ;



/* *** Read accessors *** */


inline Building* Area::get_building() const
{
  return building ;
}


inline const std::string& Area::get_name() const
{
  return name ;
}


inline const Point3D& Area::get_p_min() const
{
  return p_min ;
}


inline const Point3D& Area::get_p_max() const
{
  return p_max ;
}



/* *** Write accessors *** */


inline void Area::set_building(const Building *_building)
{
  building = const_cast<Building*>(_building) ;
}


inline void Area::set_name(const std::string &_name)
{
  name = _name ;
}


inline void Area::set_coordinates(const Point3D &p1, const Point3D &p2)
{
  p_min = p1 ;
  p_max = p2 ;
  reorder_coordinates() ;
}



/* *** Operators *** */


inline bool Area::operator!=(const Area &source) const
{
  return !(*this == source) ;
}



#endif // _OWLPS_POSITIONING_AREA_HH_
