#include "textfilereader.hh"
#include "posexcept.hh"

#include <iostream>

using namespace std ;



/* *** Constructors *** */


/**
 * @param _file_name The name of the file to open.
 * @throw error_opening_input_file if the file cannot be opened.
 */
TextFileReader::TextFileReader(const string &_file_name):
  file_name(_file_name), current_line_nb(0)
{
  if (file_name == "-")
    {
      file_buf = file.rdbuf() ;
      file.std::ios::rdbuf(std::cin.rdbuf()) ;
    }
  else
    {
      file.open(file_name.c_str()) ;
      if (! file)
        throw error_opening_input_file(file_name) ;
    }
}


TextFileReader::~TextFileReader()
{
  if (file_name == "-")
    file.std::ios::rdbuf(file_buf) ;
  else
    file.close() ;
}



/* *** Operations *** */


/**
 * Blank lines or lines containing only blank characters are skipped.
 * Tabs and spaces at the begining of a line are deleted.
 * If the first non-blank character of a line is a #, then the line is
 * skipped (note that comments at the end of a line are *not* handled).
 * @param text Output argument; unchanged in case of error.
 * @return \em false in case of error, \em true else.
 */
bool TextFileReader::read_nonblank_line(string &text)
{
  string line ;
  string::size_type first_non_blank = 0 ;

  do
    if (! read_line(line))
      return false ;
  while ((first_non_blank = line.find_first_not_of(" \t"))
         == string::npos || line.at(first_non_blank) == '#') ;

  line.erase(0, first_non_blank) ;
  text = line ;

  return true ;
}


/**
 * @return The string read, or an empty string in case of error.
 */
bool TextFileReader::read_line(string &text)
{
  if (eof_close())
    return false ;

  getline(file, text) ;
  ++current_line_nb ;
  return true ;
}


/**
 * Tests if #file is opened and there is something more to
 * read. If the end of file is reached, the stream is closed.
 * @return \em true if the end of file is reached or if the file is
 * already closed.
 * @return \em false if there is something more to read.
 */
bool TextFileReader::eof_close()
{
  if (file)
    return false ;

  if (file.eof())
    {
      file.close() ;
      return true ;
    }

  return false ;
}
