#include <cxxtest/TestSuite.h>

#include "mobile.hh"

class Mobile_test: public CxxTest::TestSuite
{
public:

  void test_constructors(void)
  {
    // Default constructor
    Mobile mobile1 ;
    Mobile mobile2("", "",
                   MOBILE_DEFAULT_ANTENNA_GAIN,
                   WIFIDEVICE_DEFAULT_TRX_POWER) ;
    TS_ASSERT_EQUALS(mobile1, mobile2) ;

    // Copy constructor
    Mobile mobile3("192.168.0.1", "aa:bb:cc:dd:ee:ff", 5, 32) ;
    Mobile mobile4(mobile3) ;
    TS_ASSERT_EQUALS(mobile3, mobile4) ;
  }


  void test_accessors(void)
  {
    // Simple read accessors
    Mobile mobile1("192.168.0.1", "aa:bb:cc:dd:ee:ff", 6, 38) ;
    TS_ASSERT_EQUALS(mobile1.get_ip_addr(), "192.168.0.1") ;
    TS_ASSERT_EQUALS(mobile1.get_mac_addr(), "aa:bb:cc:dd:ee:ff") ;
    TS_ASSERT_EQUALS(mobile1.get_antenna_gain(), 6) ;
    TS_ASSERT_EQUALS(mobile1.get_trx_power(), 38) ;

    // Write & read accessors
    mobile1.set_ip_addr("10.0.8.42") ;
    TS_ASSERT_EQUALS(mobile1.get_ip_addr(), "10.0.8.42") ;
    mobile1.set_mac_addr("00:11:22:33:44:55") ;
    TS_ASSERT_EQUALS(mobile1.get_mac_addr(), "00:11:22:33:44:55") ;
    mobile1.set_antenna_gain(12.4) ;
    TS_ASSERT_DELTA(mobile1.get_antenna_gain(), 12.4, 0.00001) ;
    mobile1.set_trx_power(22.7) ;
    TS_ASSERT_DELTA(mobile1.get_trx_power(), 22.7, 0.00001) ;
  }


  void test_operators(void)
  {
    // ==
    Mobile mobile1("192.168.0.1", "aa:bb:cc:dd:ee:ff", 6, 38) ;
    Mobile mobile2("192.168.0.1", "aa:bb:cc:dd:ee:ff", 6, 38) ;
    TS_ASSERT_EQUALS(mobile1, mobile2) ;

    // !=
    Mobile mobile3("192.168.0.2", "aa:bb:cc:dd:ee:aa", 6, 38) ;
    TS_ASSERT(mobile2 != mobile3) ;

    // =
    mobile2 = mobile3 ;
    TS_ASSERT_EQUALS(mobile2, mobile3) ;
  }

} ;
